#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May  1 13:22:21 2021

@author: mike
"""
import numpy as np
import pandas as pd

############################################
### Parameters



############################################
### Functions


def grp_ts_agg(df, grp_col, ts_col, freq_code, agg_fun, discrete=False, **kwargs):
    """
    Simple function to aggregate time series with dataframes with a single column of stations and a column of times.

    Parameters
    ----------
    df : DataFrame
        Dataframe with a datetime column.
    grp_col : str, list of str, or None
        Column name that contains the stations.
    ts_col : str
        The column name of the datetime column.
    freq_code : str
        The pandas frequency code for the aggregation (e.g. 'M', 'A-JUN').
    discrete : bool
        Is the data discrete? Will use proper resampling using linear interpolation.

    Returns
    -------
    Pandas DataFrame
    """

    df1 = df.copy()
    if isinstance(df, pd.DataFrame):
        if df[ts_col].dtype.name == 'datetime64[ns]':
            df1.set_index(ts_col, inplace=True)
            if grp_col is None:

                if discrete:
                    df3 = discrete_resample(df1, freq_code, agg_fun, **kwargs)
                else:
                    df3 = df1.resample(freq_code, **kwargs).agg(agg_fun)

                return df3

            elif isinstance(grp_col, str):
                grp_col = [grp_col]
            elif isinstance(grp_col, list):
                grp_col = grp_col[:]
            else:
                raise TypeError('grp_col must be a str, list, or None')

            if discrete:
                val_cols = [c for c in df1.columns if c not in grp_col]

                grp1 = df1.groupby(grp_col)

                grp_list = []

                for i, r in grp1:
                    s6 = discrete_resample(r[val_cols], freq_code, agg_fun, **kwargs)
                    s6[grp_col] = i

                    grp_list.append(s6)

                df2 = pd.concat(grp_list)
                df2.index.name = ts_col

                df3 = df2.reset_index().set_index(grp_col + [ts_col]).sort_index()

            else:
                grp_col.extend([pd.Grouper(level=ts_col, freq=freq_code, **kwargs)])
                df3 = df1.groupby(grp_col).agg(agg_fun)

            return df3

        else:
            raise ValueError('Make one column a timeseries!')
    else:
        raise TypeError('The object must be a DataFrame')


def discrete_resample(df, freq_code, agg_fun, remove_inter=False, **kwargs):
    """
    Function to properly set up a resampling class for discrete data. This assumes a linear interpolation between data points.

    Parameters
    ----------
    df: DataFrame or Series
        DataFrame or Series with a time index.
    freq_code: str
        Pandas frequency code. e.g. 'D'.
    agg_fun : str
        The aggregation function to be applied on the resampling object.
    **kwargs
        Any keyword args passed to Pandas resample.

    Returns
    -------
    Pandas DataFrame or Series
    """
    if isinstance(df, (pd.Series, pd.DataFrame)):
        if isinstance(df.index, pd.DatetimeIndex):
            reg1 = pd.date_range(df.index[0].ceil(freq_code), df.index[-1].floor(freq_code), freq=freq_code)
            reg2 = reg1[~reg1.isin(df.index)]
            if isinstance(df, pd.Series):
                s1 = pd.Series(np.nan, index=reg2)
            else:
                s1 = pd.DataFrame(np.nan, index=reg2, columns=df.columns)
            s2 = pd.concat([df, s1]).sort_index()
            s3 = s2.interpolate('time')
            s4 = (s3 + s3.shift(-1))/2
            s5 = s4.resample(freq_code, **kwargs).agg(agg_fun).dropna()

            if remove_inter:
                index1 = df.index.floor(freq_code).unique()
                s6 = s5[s5.index.isin(index1)].copy()
            else:
                s6 = s5
        else:
            raise ValueError('The index must be a datetimeindex')
    else:
        raise TypeError('The object must be either a DataFrame or a Series')

    return s6


def tsreg(ts, freq=None, interp=False):
    """
    Function to regularize a time series object (pandas).
    The first three indeces must be regular for freq=None!!!

    Parameters
    ----------
    ts : DataFrame
        pandas time series dataframe.
    freq : str or None
        Either specify the known frequency of the data or use None and
    determine the frequency from the first three indices.
    interp : bool
        Should linear interpolation be applied on all missing data?

    Returns
    -------
    DataFrame
    """

    if freq is None:
        freq = pd.infer_freq(ts.index[:3])
    ts1 = ts.resample(freq).mean()
    if interp:
        ts1 = ts1.interpolate('time')

    return ts1


def create_shifted_df(series, from_range, to_range, freq_code, agg_fun, ref_name, include_0=False, discrete=False, **kwargs):
    """

    """
    if not isinstance(series, pd.Series):
        raise TypeError('series must be a pandas Series.')
    if not isinstance(series.index, pd.DatetimeIndex):
        raise TypeError('The series index must be a pandas DatetimeIndex.')

    df = series.reset_index()
    data_col = df.columns[1]
    ts_col = df.columns[0]
    s2 = grp_ts_agg(df, None, ts_col, freq_code, agg_fun, discrete, **kwargs)[data_col]

    if include_0:
        f_hours = list(range(from_range-1, to_range+1))
        f_hours[0] = 0
    else:
        f_hours = list(range(from_range, to_range+1))

    df_list = []
    for d in f_hours:
        n1 = s2.shift(d, 'H')
        n1.name = ref_name + '_' + str(d)
        df_list.append(n1)
    data = pd.concat(df_list, axis=1).dropna()

    return data












































# -*- coding: utf-8 -*-
"""
Created on Wed Oct  3 16:40:35 2018

@author: michaelek
"""
import io
import os
import yaml
import pandas as pd
from tethysts import Tethys
from tethysts import utils
import copy
# from multiprocessing.pool import ThreadPool

pd.options.display.max_columns = 10

#########################################
### parameters

base_path = os.path.realpath(os.path.dirname(__file__))

with open(os.path.join(base_path, 'parameters.yml')) as param:
    param = yaml.safe_load(param)

use_type_dict = param['input']['use_type_dict']

# use_type_dict = {'Dairying - Cows': 'irrigation', 'Water Supply - Rural': 'water_supply', 'Pasture Irrigation': 'irrigation', 'Crop Irrigation': 'irrigation', 'Stock Yard': 'stockwater', 'Water Supply - Town': 'water_supply', 'Quarrying': 'other', 'Recreational': 'other', 'Gravel extraction': 'other', 'Hydro-electric power generation': 'hydro_electric', 'Food Processing': 'other', 'Meat works': 'other', 'Tourism': 'other', 'Mining works': 'other', 'Industrial': 'other', 'Domestic': 'water_supply', 'Timber Processing incl Sawmills': 'other', 'Peat Harvesting/Processing': 'other', 'Milk and dairy industries': 'other', 'Gravel Wash': 'other', 'Carwash': 'other', 'Contaminated land earthworks': 'other', 'Dairying - Sheep': 'irrigation', 'Fertiliser Work': 'other', 'Fish Processing': 'other', 'Fisheries and wildlife habitat/control': 'other', 'Food Processing': 'other', 'Gravel Wash': 'other', 'Gravel extraction': 'other', 'Horticulture Irrigation': 'irrigation', 'Landfill and transfer stations': 'other', 'Manufacturing': 'other', 'River control': 'other', 'Slink Skins': 'other', 'Stockwater': 'stockwater', 'Transport': 'other', 'Truckwash': 'other'}

#########################################
### Functions


def get_permit_data(remote):
    """

    """
    obj1 = utils.get_object_s3(**remote)
    permits = utils.read_json_zstd(obj1)

    return permits


def get_usage_data(remote, waps, from_date=None, to_date=None, threads=30):
    """

    """
    # remote = [{'bucket': bucket, 'connection_config': connection_config}]
    t1 = Tethys([remote])

    usage_ds = [ds for ds in t1.datasets if (ds['parameter'] == 'water_use') and (ds['product_code'] == 'raw_data') and (ds['frequency_interval'] == '24H') and (ds['utc_offset'] == '12H') and (ds['method'] == 'sensor_recording')]

    stns_all = []

    for ds in usage_ds:
        stns1 = t1.get_stations(ds['dataset_id'])
        stns_all.extend([s for s in stns1 if s['ref'] in waps])

    if stns_all:
        stns_dict = {s['dataset_id']: [] for s in stns_all}
        _ = [stns_dict[s['dataset_id']].extend([s['station_id']]) for s in stns_all]

        data_list = []
        for ds, stns in stns_dict.items():
            data = t1.get_results(ds, stns, from_date=from_date, to_date=to_date, squeeze_dims=True, threads=threads)

            val2 = data[['water_use', 'ref']].drop('height').to_dataframe().reset_index()
            val2 = val2.drop('geometry', axis=1).rename(columns={'ref': 'wap'}).dropna()
            data_list.append(val2)

        data2 = pd.concat(data_list)
    else:
        raise ValueError('No water use data found. Check parameters.')

    data2['time'] = data2['time'] + pd.DateOffset(hours=12)

    return data2, stns_all


def allo_filter(permits_list, from_date=None, to_date=None, permit_filter=None, wap_filter=None, only_consumptive=True, include_hydroelectric=False):
    """
    Function to filter consents and WAPs in various ways.

    Parameters
    ----------
    server : str
        The server of the Hydro db.
    from_date : str
        The start date for the time series.
    to_date: str
        The end date for the time series.
    permit_filter : dict
        If permit_id_filter is a list, then it should represent the columns from the permit table that should be returned. If it's a dict, then the keys should be the column names and the values should be the filter on those columns.
    wap_filter : dict
        If wap_filter is a list, then it should represent the columns from the wap table that should be returned. If it's a dict, then the keys should be the column names and the values should be the filter on those columns.
    only_consumptive : bool
        Should only the consumptive takes be returned? Default True
    include_hydroelectric : bool
        Should hydro-electric takes be included? Default False

    Returns
    -------
    Three DataFrames
        Representing the filters on the ExternalSites, CrcAllo, and CrcWapAllo
    """
    ### Process the premits dict into the three dataframes
    waps0 = []
    permits0 = []

    for p in permits_list:
        if p['exercised']:
            if p['activity']['activity_type'] == 'consumptive take water':
                p1 = {'permit_id': p['permit_id'], 'hydro_feature': p['activity']['feature'], 'permit_status': p['status'], 'use_type': p['activity']['primary_purpose'], 'max_rate': p['activity']['condition'][0]['limit'][0]['value'], 'from_date': p['commencement_date']}

                if 'effective_end_date' in p:
                    p1.update({'to_date': p['effective_end_date']})
                else:
                    p1.update({'to_date': p['expiry_date']})

                permits0.extend([p1])

                for s in p['activity']['station']:
                    w1 = {'permit_id': p['permit_id'], 'wap': s['ref'], 'lat': s['geometry']['coordinates'][1], 'lon': s['geometry']['coordinates'][0]}
                    if 'properties' in s:
                        if s['properties']:
                            w1.update(**s['properties'])
                    waps0.extend([w1])

    waps = pd.DataFrame(waps0)
    permits = pd.DataFrame(permits0)
    permits['from_date'] = pd.to_datetime(permits['from_date'])
    permits['to_date'] = pd.to_datetime(permits['to_date'])

    permits = permits[permits['max_rate'] > 0].copy()
    permits['max_daily_volume'] = permits['max_rate'] *60*60*24*0.001
    permits['max_annual_volume'] = permits['max_rate'] *60*60*24*365*0.001

    ### Waps
    # waps_cols = ['permit_id', 'wap', 'lon', 'lat', 'sd_ratio']
    # waps1 = waps[waps_cols].copy()

    if isinstance(wap_filter, dict):
        waps_bool1 = [waps[k].isin(v) for k, v in wap_filter.items()]
        waps_bool2 = pd.concat(waps_bool1, axis=1).prod(axis=1).astype(bool)
        waps = waps[waps_bool2].copy()

    if isinstance(wap_filter, list):
        waps = waps[waps['wap'].isin(wap_filter)].copy()

    if isinstance(permit_filter, list):
        waps = waps[waps['permit_id'].isin(permit_filter)].copy()

    ### permits
    permit_cols = ['permit_id', 'hydro_feature', 'permit_status', 'from_date', 'to_date', 'use_type', 'max_rate', 'max_daily_volume', 'max_annual_volume']

    permits1 = permits[permit_cols].copy()

    permits1['use_type'] = permits1.use_type.replace(use_type_dict)

    if isinstance(permit_filter, dict):
        permit_bool1 = [permits1[k].isin(v) for k, v in permit_filter.items()]
        permit_bool2 = pd.concat(permit_bool1, axis=1).prod(axis=1).astype(bool)
        permits1 = permits1[permit_bool2].copy()

    bool1 = True

    # if only_consumptive:
    #     bool1 = permits.exercised & permits.consumptive
    if not include_hydroelectric:
        bool1 = bool1 & (permits1.use_type != 'hydro_electric')

    permits1 = permits1.loc[bool1].copy()

    permits2 = permits1[permits1.permit_id.isin(waps.permit_id.unique())].copy()
    permits2['from_date'] = pd.to_datetime(permits2['from_date'])
    permits2['to_date'] = pd.to_datetime(permits2['to_date'])

    ## Remove consents without max rates
    permits2['max_rate'] = pd.to_numeric(permits2['max_rate'], errors='coerce')
    permits2 = permits2[permits2.max_rate.notnull()].copy()

    ## Remove consents without to/from dates or date ranges of less than a month
    permits2['from_date'] = pd.to_datetime(permits2['from_date'])
    permits2['to_date'] = pd.to_datetime(permits2['to_date'])
    permits2 = permits2[permits2['from_date'].notnull() & permits2['to_date'].notnull()]

    # Restrict dates
    if isinstance(from_date, str):
        start_time = pd.Timestamp(from_date)
        permits2 = permits2[(permits2['to_date'] - start_time).dt.days > 31]
    if isinstance(to_date, str):
        end_time = pd.Timestamp(to_date)
        permits2 = permits2[(end_time - permits2['from_date']).dt.days > 31]

    permits3 = permits2[(permits2['to_date'] - permits2['from_date']).dt.days > 31].copy()

    ## Calculate rates, daily and annual volumes (if they don't exist)
    # permits3.loc[permits3.max_rate.isnull(), 'max_rate'] = (permits3.loc[permits3.max_rate.isnull(), 'max_daily_volume'] * 1000/60/60/24).round()

    # permits3.loc[permits3.max_daily_volume.isnull(), 'max_daily_volume'] = (permits3.loc[permits3.max_daily_volume.isnull(), 'max_rate'] * 60*60*24/1000).round()

    # permits3.loc[permits3.max_annual_volume.isnull(), 'max_annual_volume'] = (permits3.loc[permits3.max_annual_volume.isnull(), 'max_daily_volume'] * 365).round()

    ## Index by permit_id and hydro_group - keep the largest limits
    limit_cols = ['max_rate', 'max_daily_volume', 'max_annual_volume']
    other_cols = list(permits3.columns[~(permits3.columns.isin(limit_cols) | permits3.columns.isin(['permit_id', 'hydro_feature']))])
    grp1 = permits3.groupby(['permit_id', 'hydro_feature'])
    other_df = grp1[other_cols].first()
    limits_df = grp1[limit_cols].max()

    permits4 = pd.concat([other_df, limits_df], axis=1).reset_index()

    ### sd
    # sd_cols = ['wap', 'sd_ratio']

    # sd1 = sd[sd_cols].copy()

    ### Filter
    # sd2 = sd1[(sd1.permit_id.isin(permits3.permit_id.unique())) & (sd1.wap.isin(waps1.wap.unique()))].copy()

    permits5 = permits4[permits4.permit_id.isin(waps.permit_id.unique())].copy()
    waps2 = waps[waps.permit_id.isin(permits5.permit_id.unique())].copy()

    ### Index the DataFrames
    # permit_id_allo2.set_index(['permit_id', 'hydro_group'], inplace=True)
    # permit_id_wap2.set_index(['permit_id', 'hydro_group', 'wap'], inplace=True)
    # sites2.set_index('wap', inplace=True)

    return waps2, permits5




####################################################
### Testing

# remote = param['remote']
# #
# usage_conn_config = remote['usage']['connection_config']
# permit_conn_config = remote['permit']['connection_config']
# waps_key = remote['permit']['waps_key']
# permits_key = remote['permit']['permits_key']
# sd_key = remote['permit']['sd_key']
# permit_bucket = remote['permit']['bucket']
# usage_bucket = remote['usage']['bucket']
#
#
# waps, permits, sd = get_permit_data(permit_conn_config, permit_bucket, waps_key, permits_key, sd_key)
#
#
# waps, permits, sd = allo_filter(waps, permits, sd)


# waps = ['D45/0069', 'F45/0472', 'E45/0078', 'SW/0019', 'SW/0079']
#
#
# use_data = get_usage_data(connection_config, bucket, waps)



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 18 09:13:34 2019

@author: michaelek
"""
import os
from allotools import AlloUsage
from pdsql import mssql


#######################################
### Parameters

server = 'sql2012test01'
database = 'hydro'
sites_table = 'ExternalSite'

catch_group = ['Ashburton River']
summ_col = 'SwazName'

waitaki = {'CatchmentGroupName': ['Waitaki']}

crc_filter = {'use_type': ['stockwater', 'irrigation']}

from_date = '2015-07-01'
to_date = '2018-06-30'

datasets = ['allo', 'restr_allo', 'metered_allo', 'metered_restr_allo', 'usage']
datasets = ['allo', 'metered_allo', 'usage']

freq = 'A-JUN'
#freq = 'M'

t1 = 'CRC012123'
t2 = 'Pendarves Area'

t3 = 'CRC011245'

cols = ['SwazName', 'use_type', 'date']

export_path = r'E:\ecan\local\Projects\requests\Ilja\2019-02-27'


########################################
### Test 1
sites1 = mssql.rd_sql(server, database, sites_table, ['ExtSiteID', 'CatchmentGroupName', summ_col], where_in={'CatchmentGroupName': catch_group})

site_filter = {'SwazName': sites1.SwazName.unique().tolist()}

a1 = AlloUsage(from_date, to_date, site_filter=site_filter, crc_filter=crc_filter)

allo_ts1 = a1._get_allo_ts(freq, groupby=['crc', 'date'])

metered_allo_ts1 = a1._get_metered_allo_ts(freq, ['crc', 'date'])

usage1 = a1._get_usage_ts(freq, ['crc', 'date'])

a1._lowflow_data(freq)

restr_allo = a1._get_restr_allo_ts(freq, ['crc', 'wap', 'date'])

combo_ts1 = a1.get_ts(datasets, freq, ['date'])
combo_ts2 = a1.get_ts(datasets, freq, ['date'], irr_season=True)

combo_ts1.resample('A-JUN').sum().round(-4)
combo_ts2.resample('A-JUN').sum().round(-4)

### Test 2
a1 = AlloUsage(from_date, to_date)

combo_ts = a1.get_ts(datasets, freq, ['date'])

a1 = AlloUsage(from_date, to_date, crc_filter={'crc': ['CRC157373']})

a1 = AlloUsage(from_date, to_date, crc_filter={'crc': ['CRC182351']})
combo_ts = a1.get_combo_ts(datasets, freq, ['date'])


### Test3
a1 = AlloUsage(from_date, to_date, crc_filter={'crc': [t3]})

combo_ts1 = a1.get_ts(datasets, freq, cols)
combo_ts2 = a1.get_ts(datasets, freq, ['date'], irr_season=True)

c1 = combo_ts1.resample('A-JUN').sum().round(-4)[['total_allo', 'total_metered_allo', 'total_restr_allo', 'total_metered_restr_allo', 'total_usage']]
c2 = combo_ts2.resample('A-JUN').sum().round(-4)[['total_allo', 'total_metered_allo', 'total_restr_allo', 'total_metered_restr_allo', 'total_usage']]

c1['total_restr_allo']/c1['total_allo']
c2['total_restr_allo']/c2['total_allo']


combo_ts3 = a1.get_ts(datasets, 'A-JUN', ['date'], irr_season=True)[['total_allo', 'total_metered_allo', 'total_restr_allo', 'total_metered_restr_allo', 'total_usage']].round(-4)

### Test 4
sites1 = mssql.rd_sql(server, database, sites_table, ['ExtSiteID', 'CatchmentGroupName', summ_col], where_in={'CatchmentGroupName': catch_group})

site_filter = {'SwazName': sites1.SwazName.unique().tolist()}

a1 = AlloUsage(from_date, to_date, site_filter=site_filter)

ts1 = a1.get_ts(datasets, freq, cols, irr_season=True)

a1.plot_group('A-JUN', val='total', with_restr=True, export_path=r'E:\ecan\local\Projects\requests\suz\2018-12-17\plots')

a1.plot_stacked('A-JUN', val='total', export_path=r'E:\ecan\local\Projects\requests\suz\2018-12-17\plots')

### Test 5
a1 = AlloUsage(from_date, to_date, site_filter=waitaki)

a1.allo.to_csv(os.path.join(export_path, 'waitaki_allo_2019-02-27.csv'))

a1.allo_wap.to_csv(os.path.join(export_path, 'waitaki_allo_wap_2019-02-27.csv'))
















# -*- coding: utf-8 -*-
"""
Created on Wed Jun 29 13:57:22 2016

@author: Kate and Mike
"""


def allo_band_plt(csv_path, river):

    from pandas import read_csv
    import matplotlib.pyplot as plt
    matplotlib.style.use('ggplot')

    Alloc = read_csv(csv_path, index_col=(0), header=[0,1], skiprows=[2])
    Alloc = Alloc.fillna(0)
    Alloc = Alloc.rename(columns={'S&D no min flow': 'Stock and domestic, no minimum flow'})

    #Irrigation use and allocation band plot
    Alloc1 = Alloc.sum(axis=1, level=1)

    AllocPlot1 = Alloc1.plot.bar(stacked=True,colormap='Set3')
    AllocPlot1 = plt.xlabel('Year')
    AllocPlot1 = plt.ylabel('Allocation (L/s)')
    AllocPlot1 = plt.title(river + ' allocation')
    AllocPlot2 = plt.legend(loc='upper left', title="Water use & allocation band")

    #Irrigation source plot
    Alloc2 = Alloc.sum(axis=1, level=0)

    AllocPlot2 = Alloc2.plot.area(stacked=True)
    AllocPlot2 = plt.xlabel('Year')
    AllocPlot2 = plt.ylabel('Allocation (L/s)')
    AllocPlot2 = plt.title(river + ' allocation')
    AllocPlot2 = plt.legend(["Groundwater", "Surface water"], loc='upper left', title="Water source")

    return(AllocPlot1, AllocPlot2)


def allo_plt(df, yaxis_mag=1000000, yaxis_lab='Million', start='2006', end='2015', cat=['tot_allo', 'meter_allo', 'meter_usage'], col_pal='pastel', export_path='', export_name='tot_allo_use_restr.png'):
    """
    Function to plot either total allocation with restrictions or total allo, metered allo, and metered usage with restrictions over a period of years.
    """
    from os import path
    from numpy import in1d
    import seaborn as sns
    from pandas import to_datetime, melt, Period
    import matplotlib.pyplot as plt
#    import matplotlib.patheffects as pathe
#    import matplotlib.ticker as ticker
#    import matplotlib.dates as mdates

    ### Reorganize data
    allo1 = df.sum(axis=0, level=0)
    allo1.index = to_datetime(allo1.index)
    allo1.index.name = 'date'
    allo2 = allo1[start:end] * 1 / yaxis_mag
    dict1 = {'tot_allo': 'tot_allo', 'meter_allo': 'allo', 'meter_usage': 'usage'}
#    dict2 = {'tot_allo': 'tot_ann_up_allo_m3', 'meter_allo': 'ann_up_allo_m3', 'meter_usage': 'usage_m3'}
    lst1 = [dict1[d] for d in cat]
#    lst2 = [dict2[d] for d in cat]

    if allo2.size > 1:
#        index1 = allo2.index.astype('str').str[0:4].astype('int')
#        allo2['index'] = index2

        pw = len(str(yaxis_mag)) - 1

        allo_all = melt(allo2.reset_index(), id_vars='date', value_vars=['tot_allo', 'allo', 'usage'], var_name='tot_allo')
#        allo_up_all = melt(allo2.reset_index(), id_vars='dates', value_vars=['tot_ann_up_allo_m3', 'ann_up_allo_m3', 'usage_m3'], var_name='up_allo')
#        allo_up_all.loc[allo_up_all.up_allo == 'usage_m3', 'value'] = 0

        allo_all = allo_all[in1d(allo_all.tot_allo, lst1)]
#        allo_up_all = allo_up_all[in1d(allo_up_all.up_allo, lst2)]

        index1 = allo_all.date.astype('str').str[0:4].astype('int')
        index2 = [Period(d) for d in index1.tolist()]

        ### Total Allo and restricted allo and usage
        ## Set basic plot settings
        sns.set_style("whitegrid")
        sns.set_context('poster')

        ## Plot total allo
        fig, ax = plt.subplots(figsize=(15, 10))
        sns.barplot(x=index2, y='value', hue='tot_allo', data=allo_all, palette=col_pal, edgecolor='0')
#        sns.barplot(x=index2, y='value', hue='up_allo', data=allo_up_all, palette=col_pal, edgecolor='0', hatch='/')
#        plt.ylabel('Water Volume $(10^{' + str(pw) + '} m^{3}/year$)')
        plt.ylabel('Water Volume $(' + yaxis_lab + '\; m^{3}/year$)')
        plt.xlabel('Year')

        # Legend
        if len(cat) == 3:
            handles, lbs = ax.get_legend_handles_labels()
            order1 = [0,3,1,4,2]
            labels = ['Total allocation', 'Total allocation with restrictions', 'Metered allocation', 'Metered allocation with restrictions', 'Metered usage']
            leg1 = plt.legend([handles[i] for i in order1], labels, loc='upper left')
#        leg1.legendPatch.set_path_effects(pathe.withStroke(linewidth=5, foreground="w"))
        else:
            handles, lbs = ax.get_legend_handles_labels()
            order1 = [0,3]
            plt.legend(handles, ['Total allocation', 'Total allocation with restrictions'], loc='upper left')

        xticks = ax.get_xticks()
        if len(xticks) > 15:
            for label in ax.get_xticklabels()[::2]:
                label.set_visible(False)
            ax.xaxis_date()
            fig.autofmt_xdate(ha='center')
            plt.tight_layout()
        plt.tight_layout()
#      sns.despine(offset=10, trim=True)
        plot2 = ax.get_figure()
        plot2.savefig(path.join(export_path, export_name))

        ### Return plot
        return(ax)


def allo_multi_plot(df, agg_level=[0, 1], index_level=1, plot_fun=allo_plt, export_name='tot_allo_use_restr.png', export_path='', **kwargs):
    """
    Function that calls allo_plt to plot summarized groups from the allo query into several plots.
    """
#    from allo_use_plot_fun import allo_plt

    ### Aggregate to specific level
    allo1 = df.sum(axis=0, level=agg_level)
#    index_name = allo1.index.names[agg_level]
    allo2 = allo1.reset_index()
    index_unique = allo2.ix[:, index_level].unique()

    ### Loop through the index
    for i in index_unique:
        df2 = allo1.xs(i, level=index_level)
        plot_fun(df2, export_name=i + '_' + export_name, export_path=export_path, **kwargs)


def allo_stacked_plt(df, yaxis_mag=1000000, yaxis_lab='Million', start='1990', end='2015', agg_level=[0, 1], cat=['tot_allo'], cat_type='use_type', col_pal='pastel', export_path='', export_name='tot_allo_type.png'):
    """
    Function to plot the summarized groups as stacked bars of the total.
    """
    from os import path
    from numpy import in1d, where, array
    import seaborn as sns
    from pandas import to_datetime, melt, Period
    import matplotlib.pyplot as plt

    ### Reorganize data
    ## Set up dictionaries and parameters
    dict_type = {'public_supply': 'Public Water Supply', 'irrigation': 'Irrigation', 'stockwater': 'Stockwater', 'other': 'Other', 'industry': 'Industry'}
    order1 = array(['public_supply', 'irrigation', 'stockwater', 'other', 'industry'])
    cols1 = sns.color_palette(col_pal)
    col_dict = {'public_supply': cols1[0], 'irrigation': cols1[1], 'stockwater': cols1[2], 'other': cols1[4], 'industry': cols1[3]}

    df2 = df[['tot_ann_allo_m3', 'ann_allo_m3', 'usage_m3']]
    df2 = df2.sum(axis=0, level=agg_level)
    df3 = df2.stack()
    if cat_type is 'use_type':
        df4 = df3.unstack(1)
        cols1 = order1[in1d(order1, df4.columns)]
        df4 = df4[cols1]
        dict0 = dict_type
    else:
        df4 = df3.unstack(1)
    df5 = df4.cumsum(axis=1)
    allo1 = df5.unstack()

    grp1 = df4.columns.tolist()
    lab_names = [dict0[i] for i in grp1]
    col_lab = [col_dict[i] for i in grp1]

    allo1.index = to_datetime(allo1.index)
    allo1.index.name = 'dates'
    allo2 = allo1[start:end] * 1 / yaxis_mag
    dict1 = {'tot_allo': 'tot_ann_allo_m3', 'meter_allo': 'ann_allo_m3', 'meter_usage': 'usage_m3'}
    lst1 = [dict1[d] for d in cat]

#    colors = sns.color_palette(col_pal)

    if allo2.size > 1:
        ### Set plotting parameters
        sns.set_style("whitegrid")
        sns.set_context('poster')
        pw = len(str(yaxis_mag)) - 1

        fig, ax = plt.subplots(figsize=(15, 10))

        for i in grp1[::-1]:
            seq1 = int(where(in1d(grp1, i))[0])
            allo_all = melt(allo2[i].reset_index(), id_vars='dates', value_vars=['tot_ann_allo_m3', 'ann_allo_m3', 'usage_m3'], var_name=i)
            allo_all = allo_all[in1d(allo_all[i], lst1)]

            index1 = allo_all.dates.astype('str').str[0:4].astype('int')
            index2 = [Period(d) for d in index1.tolist()]
            sns.barplot(x=index2, y='value', data=allo_all, edgecolor='0', color=col_lab[seq1], label=i)

#        plt.ylabel('Allocated Water Volume $(10^{' + str(pw) + '} m^{3}/year$)')
        plt.ylabel('Water Volume $(' + yaxis_lab + '\; m^{3}/year$)')
        plt.xlabel('Year')

        # Legend
        handles, lbs = ax.get_legend_handles_labels()
        plt.legend(handles, lab_names[::-1], loc='upper left')

        xticks = ax.get_xticks()
        if len(xticks) > 15:
            for label in ax.get_xticklabels()[::2]:
                label.set_visible(False)
            ax.xaxis_date()
            fig.autofmt_xdate(ha='center')
            plt.tight_layout()
        plt.tight_layout()
#      sns.despine(offset=10, trim=True)
        plot2 = ax.get_figure()
        plot2.savefig(path.join(export_path, export_name))

        ### Return plot
        return(ax)


def allo_restr_plt(df, yaxis_mag=1000000, yaxis_lab='Million', start='2006', end='2015', cat=['tot_allo', 'meter_allo', 'meter_usage'], col_pal='pastel', export_path='', export_name='tot_allo_use_restr.png'):
    """
    Function to plot either total allocation with restrictions or total allo, metered allo, and metered usage with restrictions over a period of years.
    """
    from os import path
    from numpy import in1d
    import seaborn as sns
    from pandas import to_datetime, melt, Period
    import matplotlib.pyplot as plt
    from collections import OrderedDict
#    import matplotlib.patheffects as pathe
#    import matplotlib.ticker as ticker
#    import matplotlib.dates as mdates

    base_cat = ['tot_allo', 'meter_allo', 'meter_usage']

    ### Reorganize data
    allo1 = df.sum(axis=0, level=0)
    allo1.index = to_datetime(allo1.index)
    allo1.index.name = 'date'
    allo2 = allo1[start:end] * 1 / yaxis_mag
    dict1 = {'tot_allo': 'tot_allo', 'meter_allo': 'allo', 'meter_usage': 'usage'}
    dict2 = {'tot_allo': 'tot_allo_restr', 'meter_allo': 'allo_restr', 'meter_usage': 'usage'}
    lst1 = [dict1[d] for d in cat]
    lst2 = [dict2[d] for d in cat]

    if allo2.size > 1:
#        index1 = allo2.index.astype('str').str[0:4].astype('int')
#        allo2['index'] = index2

        pw = len(str(yaxis_mag)) - 1

        allo_all = melt(allo2.reset_index(), id_vars='date', value_vars=['tot_allo', 'allo', 'usage'], var_name='tot_allo')
        allo_up_all = melt(allo2.reset_index(), id_vars='dates', value_vars=['tot_allo_restr', 'allo_restr', 'usage'], var_name='up_allo')
        allo_up_all.loc[allo_up_all.up_allo == 'usage', 'value'] = 0

        allo_all = allo_all[in1d(allo_all.tot_allo, lst1)]
        allo_up_all = allo_up_all[in1d(allo_up_all.up_allo, lst2)]

        index1 = allo_all.date.astype('str').str[0:4].astype('int')
        index2 = [Period(d) for d in index1.tolist()]

        ### Total Allo and restricted allo and usage
        ## Set basic plot settings
        sns.set_style("whitegrid")
        sns.set_context('poster')
        col_pal1 = sns.color_palette(col_pal)
        col_pal2 = [col_pal1[base_cat.index(i)] for i in cat]

        ## Plot total allo
        fig, ax = plt.subplots(figsize=(15, 10))
        sns.barplot(x=index2, y='value', hue='tot_allo', data=allo_all, palette=col_pal2, edgecolor='0')
        sns.barplot(x=index2, y='value', hue='up_allo', data=allo_up_all, palette=col_pal2, edgecolor='0', hatch='/')
#        plt.ylabel('Water Volume $(10^{' + str(pw) + '} m^{3}/year$)')
        plt.ylabel('Water Volume $(' + yaxis_lab + '\; m^{3}/year$)')
        plt.xlabel('Year')

        # Legend
        handles, lbs = ax.get_legend_handles_labels()
        hand_len = len(handles)
        order1 = [lbs.index(j) for j in ['tot_allo', 'tot_allo_restr', 'allo', 'allo_restr', 'usage'] if j in lbs]
        label_dict1 = OrderedDict([('tot_allo', ['Total allocation', 'Total allocation with restrictions']), ('meter_allo', ['Metered allocation', 'Metered allocation with restrictions']), ('meter_usage', ['Metered usage'])])
        labels = []
        [labels.extend(label_dict1[i]) for i in cat]
        leg1 = plt.legend([handles[i] for i in order1], labels, loc='upper left')
#        leg1.legendPatch.set_path_effects(pathe.withStroke(linewidth=5, foreground="w"))

        xticks = ax.get_xticks()
        if len(xticks) > 15:
            for label in ax.get_xticklabels()[::2]:
                label.set_visible(False)
            ax.xaxis_date()
            fig.autofmt_xdate(ha='center')
            plt.tight_layout()
        plt.tight_layout()
#      sns.despine(offset=10, trim=True)
        plot2 = ax.get_figure()
        plot2.savefig(path.join(export_path, export_name))

        ### Return plot
        return(ax)













